==> otbnos-3.0.zip: otamesiBitNOS/bitnos/driver/mouse/mouse.cpp <==
#include "../../bootpack.h"

/*
 * statico
 */
bool Mouse::scroll_ = false;
MouseDecode Mouse::mdec;

void Mouse::init_cursor(system::Picture& cursor) {
	static const char mcursor1[YSIZE][XSIZE + 1] = {
		"*___________",
		"**__________",
		"*.*_________",
		"*..*________",
		"*...*_______",
		"*....*______",
		"*.....*_____",
		"*......*____",
		"*.......*___",
		"*........*__",
		"*.........*_",
		"*......*****",
		"*...*..*____",
		"*..**..*____",
		"*.*__*..*___",
		"**___*..*___",
		"*_____*..*__",
		"______*..*__",
		"_______*..*_",
		"_______*..*_",
		"________***_",
	};
	int y, x;
	for (y = 0; y < YSIZE; y++) {
		for (x = 0; x < XSIZE; x++) {
			if (mcursor1[y][x] == '*') {
				cursor.point(x, y, 0x0000);
			} else if (mcursor1[y][x] == '.') {
				cursor.point(x, y, 0xffff);
			} else {
				cursor.point(x, y, 0x0001);
			}
		}
	}
}

void Mouse::enable() {
	#ifndef DEBUG_MESSAGE
	Message* msg;
	#else
	Message* msg;
	Message msg_;
	#endif
	int i = 0;
	int errors = 0; // G[̉
	bool send = false; // }EX֑Mtrue
	
	// }EXzC[L
	for (;;) {
		/*
		 * ܂}EX֐MM
		 */
		if (i == 0 && send == false) {
			out(0xf3);
			send = true;
		} else if (i == 1 && send == false) {
			out(200);
			send = true;
		} else if (i == 2 && send == false) {
			out(0xf3);
			send = true;
		} else if (i == 3 && send == false) {
			out(100);
			send = true;
		} else if (i == 4 && send == false) {
			out(0xf3);
			send = true;
		} else if (i == 5 && send == false) {
			out(80);
			send = true;
		} else if (i == 6 && send == false) {
			out(0xf2);
			send = true;
		}
		io_cli();
		if (task1_fifo->size() == 0) {
			io_stihlt();
			continue;
		}
		#ifndef DEBUG_MESSAGE
		msg = task1_fifo->back();
		#else
		msg_ = task1_fifo->back();
		msg = &msg_;
		#endif
		task1_fifo->pop();
		io_sti();
		
		if (msg->type == MSG::TYPE::MOUSE) {
		
			if (0 <= i && i <= 6 && msg->arg1 == 0xfa) {
				// G[
			} else if (i == 7 && msg->arg1 == 0) {
				// zC[
				scroll_ = false;
			} else if (i == 7 && msg->arg1 == 3) {
				// zC[L
				scroll_ = true;
			}
			
			if (i == 7 || errors > 10) {
				/*
				 * [v𔲂
				 */
				#ifndef DEBUG_MESSAGE
				delete msg;
				#endif
				break;
			}
			
			if (msg->arg1 != 0xfe) {
				// đvł͖
				i++;
				send = false;
			} else {
				errors++;
			}
		}
		
		#ifndef DEBUG_MESSAGE
		delete msg;
		#endif
	}
	
	// }EX
	for (i = 0, send = false;;) {
		/*
		 * }EX̏
		 */
		if (i == 0 && send == false) {
			out(0xe8); // set resolution
			send = true;
		} else if (i == 1 && send == false) {
			out(0x03); // 8 count/mm
			send = true;
		} else if (i == 2 && send == false) {
			out(0xe6); // set scaling 1:1
			send = true;
		} else if (i == 3 && send == false) {
			out(0xf3); // set sample rate
			send = true;
		} else if (i == 4 && send == false) {
			out(40);
			send = true;
		}
		io_cli();
		if (task1_fifo->size() == 0) {
			io_stihlt();
			continue;
		}
		#ifndef DEBUG_MESSAGE
		msg = task1_fifo->back();
		#else
		msg_ = task1_fifo->back();
		msg = &msg_;
		#endif
		task1_fifo->pop();
		io_sti();
		
		if (msg->type == MSG::TYPE::MOUSE) {
		
			if (0 <= i && i <= 4 && msg->arg1 == 0xfa) {
				// G[
			}
			
			if (i == 4 || errors > 10) {
				/*
				 * }EXÎŃ[v𔲂
				 */
				#ifndef DEBUG_MESSAGE
				delete msg;
				#endif
				break;
			}
			
			if (msg->arg1 != 0xfe) {
				// đvł͖
				i++;
				send = false;
			} else {
				errors++;
			}
		}
		
		#ifndef DEBUG_MESSAGE
		delete msg;
		#endif
	}

	// ȉsƁA}EX̍W񂪓͂悤ɂȂ
	out(MOUSECMD_ENABLE);
	
	// }EX0xfâ҂iK
	mdec.phase = 0;
}

bool Mouse::decode(byte data) {
	bool finished = false; // ͂Itrue
	if (mdec.phase == 0) {
		if (data == 0xfa) {
			mdec.phase = 1;
		}
	} else if (mdec.phase == 1) {
		if ((data & 0xc8) == 0x08) {
			// 1oCgڂ
			mdec.buf[0] = data;
			mdec.phase++;
		}
	} else if (mdec.phase == 2) {
		mdec.buf[1] = data;
		mdec.phase++;
	} else if (mdec.phase == 3) {
		mdec.buf[2] = data;
		if (scroll_ == true) {
			// }EX̓zC[L
			mdec.phase++;
		} else {
			// }EX̓zC[
			mdec.phase = 1;
			
			// zC[Ȃ̂ŁAȉ̉͂IOK
			finished = true;
		}
		
		// ꂩ}EXf[^̉͂
		// ̉͂́AzC[LAŋʂł
		mdec.button = mdec.buf[0] & 0x07;
		mdec.x = mdec.buf[1];
		mdec.y = mdec.buf[2];
		
		if (mdec.buf[0] & 0x10) {
			// X sign bit == 1
			mdec.x |= 0xffffff00;
		}
		
		if (mdec.buf[0] & 0x20) {
			// Y sign bit == 1
			mdec.y |= 0xffffff00;
		}
		mdec.y = -mdec.y;
	} else if (mdec.phase == 4) {
		mdec.buf[3] = data;
		mdec.phase = 1;
		finished = true;
		
		// mdec.buf[3]́A4rbgLȒlł
		// Ƃ肠͂ɒl܂B
		mdec.scroll = mdec.buf[3] & 0x0f;
		if (mdec.scroll & 0x08) {
			// }CiX̒l
			mdec.scroll |= 0xfffffff0;
		}
	}
	return finished;
}

void Mouse::out(byte data)
{
	IO8 keycmd(Keyboard::PORT_KEYCMD);
	IO8 keydat(Keyboard::PORT_KEYDAT);
	Keyboard::wait_sendready();
	keycmd << Mouse::KEYCMD_SENDTO_MOUSE;
	Keyboard::wait_sendready();
	keydat << data;
}

==> otbnos-3.1.zip: otamesiBitNOS/bitnos/driver/mouse/mouse.cpp <==
#include "../../bootpack.h"

/*
 * statico
 */
bool Mouse::scroll_ = false;
MouseDecode Mouse::mdec;

void Mouse::init_cursor(system::Picture* cursor) {
	static const char mcursor1[YSIZE][XSIZE + 1] = {
		"*___________",
		"**__________",
		"*.*_________",
		"*..*________",
		"*...*_______",
		"*....*______",
		"*.....*_____",
		"*......*____",
		"*.......*___",
		"*........*__",
		"*.........*_",
		"*......*****",
		"*...*..*____",
		"*..**..*____",
		"*.*__*..*___",
		"**___*..*___",
		"*_____*..*__",
		"______*..*__",
		"_______*..*_",
		"_______*..*_",
		"________***_",
	};
	int y, x;
	for (y = 0; y < YSIZE; y++) {
		for (x = 0; x < XSIZE; x++) {
			if (mcursor1[y][x] == '*') {
				cursor->point(x, y, 0x0000);
			} else if (mcursor1[y][x] == '.') {
				cursor->point(x, y, 0xffff);
			} else {
				cursor->point(x, y, 0x0001);
			}
		}
	}
}

void Mouse::enable(system::Task* task) {
	#ifndef DEBUG_MESSAGE
	Message* msg;
	#else
	Message* msg;
	Message msg_;
	#endif
	int i = 0;
	int errors = 0; // G[̉
	bool send = false; // }EX֑Mtrue
	queue<Message>* fifo = task->fifo();
	
	// }EXzC[L
	for (;;) {
		/*
		 * ܂}EX֐MM
		 */
		if (i == 0 && send == false) {
			out(0xf3);
			send = true;
		} else if (i == 1 && send == false) {
			out(200);
			send = true;
		} else if (i == 2 && send == false) {
			out(0xf3);
			send = true;
		} else if (i == 3 && send == false) {
			out(100);
			send = true;
		} else if (i == 4 && send == false) {
			out(0xf3);
			send = true;
		} else if (i == 5 && send == false) {
			out(80);
			send = true;
		} else if (i == 6 && send == false) {
			out(0xf2);
			send = true;
		}
		io_cli();
		if (fifo->size() == 0) {
			g_taskctl->sleep(task);
			io_sti();
			continue;
		}
		#ifndef DEBUG_MESSAGE
		msg = task1_fifo->back();
		#else
		msg_ = fifo->back();
		msg = &msg_;
		#endif
		io_sti();
		
		if (msg->type == MSG::TYPE::MOUSE) {
		
			if (0 <= i && i <= 6 && msg->arg1 == 0xfa) {
				// G[
			} else if (i == 7 && msg->arg1 == 0) {
				// zC[
				scroll_ = false;
			} else if (i == 7 && msg->arg1 == 3) {
				// zC[L
				scroll_ = true;
			}
			
			if (i == 7 || errors > 10) {
				/*
				 * [v𔲂
				 */
				#ifndef DEBUG_MESSAGE
				delete msg;
				#endif
				break;
			}
			
			if (msg->arg1 != 0xfe) {
				// đvł͖
				i++;
				send = false;
			} else {
				errors++;
			}
		}
		
		#ifndef DEBUG_MESSAGE
		delete msg;
		#endif
		fifo->pop();
	}
	
	// }EX
	for (i = 0, send = false;;) {
		/*
		 * }EX̏
		 */
		if (i == 0 && send == false) {
			out(0xe8); // set resolution
			send = true;
		} else if (i == 1 && send == false) {
			out(0x03); // 8 count/mm
			send = true;
		} else if (i == 2 && send == false) {
			out(0xe6); // set scaling 1:1
			send = true;
		} else if (i == 3 && send == false) {
			out(0xf3); // set sample rate
			send = true;
		} else if (i == 4 && send == false) {
			out(40);
			send = true;
		}
		io_cli();
		if (fifo->size() == 0) {
			g_taskctl->sleep(task);
			io_sti();
			continue;
		}
		#ifndef DEBUG_MESSAGE
		msg = task1_fifo->back();
		#else
		msg_ = fifo->back();
		msg = &msg_;
		#endif
		io_sti();
		
		if (msg->type == MSG::TYPE::MOUSE) {
		
			if (0 <= i && i <= 4 && msg->arg1 == 0xfa) {
				// G[
			}
			
			if (i == 4 || errors > 10) {
				/*
				 * }EXÎŃ[v𔲂
				 */
				#ifndef DEBUG_MESSAGE
				delete msg;
				#endif
				break;
			}
			
			if (msg->arg1 != 0xfe) {
				// đvł͖
				i++;
				send = false;
			} else {
				errors++;
			}
		}
		
		#ifndef DEBUG_MESSAGE
		delete msg;
		#endif
		fifo->pop();
	}

	// ȉsƁA}EX̍W񂪓͂悤ɂȂ
	out(MOUSECMD_ENABLE);
	
	// }EX0xfâ҂iK
	mdec.phase = 0;
}

bool Mouse::decode(byte data) {
	bool finished = false; // ͂Itrue
	if (mdec.phase == 0) {
		if (data == 0xfa) {
			mdec.phase = 1;
		}
	} else if (mdec.phase == 1) {
		if ((data & 0xc8) == 0x08) {
			// 1oCgڂ
			mdec.buf[0] = data;
			mdec.phase++;
		}
	} else if (mdec.phase == 2) {
		mdec.buf[1] = data;
		mdec.phase++;
	} else if (mdec.phase == 3) {
		mdec.buf[2] = data;
		if (scroll_ == true) {
			// }EX̓zC[L
			mdec.phase++;
		} else {
			// }EX̓zC[
			mdec.phase = 1;
			
			// zC[Ȃ̂ŁAȉ̉͂IOK
			finished = true;
		}
		
		// ꂩ}EXf[^̉͂
		// ̉͂́AzC[LAŋʂł
		mdec.button = mdec.buf[0] & 0x07;
		mdec.x = mdec.buf[1];
		mdec.y = mdec.buf[2];
		
		if (mdec.buf[0] & 0x10) {
			// X sign bit == 1
			mdec.x |= 0xffffff00;
		}
		
		if (mdec.buf[0] & 0x20) {
			// Y sign bit == 1
			mdec.y |= 0xffffff00;
		}
		mdec.y = -mdec.y;
	} else if (mdec.phase == 4) {
		mdec.buf[3] = data;
		mdec.phase = 1;
		finished = true;
		
		// mdec.buf[3]́A4rbgLȒlł
		// Ƃ肠͂ɒl܂B
		mdec.scroll = mdec.buf[3] & 0x0f;
		if (mdec.scroll & 0x08) {
			// }CiX̒l
			mdec.scroll |= 0xfffffff0;
		}
	}
	return finished;
}

void Mouse::out(byte data)
{
	IO8 keycmd(Keyboard::PORT_KEYCMD);
	IO8 keydat(Keyboard::PORT_KEYDAT);
	Keyboard::wait_sendready();
	keycmd << Mouse::KEYCMD_SENDTO_MOUSE;
	Keyboard::wait_sendready();
	keydat << data;
}
